@@ -2,7 +2,7 @@
 # 
 # Make simple Subversion revision merges and branch switching much easier.
 #
-# Copyright (c) 2006--2008 Philipp von Weitershausen, Marius Gedminas
+# Copyright (c) 2006--2009 Philipp von Weitershausen, Marius Gedminas
 #
 # This program is distributed under the terms of the GNU General Public Licence
 # See the file COPYING for details.
@@ -17,11 +17,11 @@
 import os
 import sys
 import optparse
-import popen2 # TODO: use subprocess
+import subprocess
 from xml.dom import minidom
 
 
-VERSION = '1.8.0'
+VERSION = '1.10.0'
 
 
 #
@@ -95,25 +95,25 @@ def svninfo(path):
     """
     Return svn information about ``path``.
     """
-    stdout, stdin = popen2.popen2('svn info %s' % path)
-    return stdout.read()
+    p = subprocess.Popen(['svn', 'info', path], stdout=subprocess.PIPE)
+    return p.communicate()[0]
 
 
 def svnls(url):
     """
     Return a list of files under ``url``.
     """
-    stdout, stdin = popen2.popen2('svn ls %s' % url)
-    return stdout.read()
+    p = subprocess.Popen(['svn', 'ls', url], stdout=subprocess.PIPE)
+    return p.communicate()[0]
 
 
 def svnlog(url):
     """
     Return svn log of ``url``, stopping on branchpoints, in XML.
     """
-    stdout, stdin = popen2.popen2('svn log --non-interactive --stop-on-copy'
-                                  ' --xml %s' % url)
-    return stdout.read()
+    p = subprocess.Popen(['svn', 'log', '--non-interactive', '--stop-on-copy',
+                          '--xml', url], stdout=subprocess.PIPE)
+    return p.communicate()[0]
 
 
 def currentbranch(path, svninfo=svninfo):
@@ -196,6 +196,20 @@ def determinebranch(branch, path, svninfo=svninfo):
       >>> determinebranch('trunk', '.', svninfo=dummyinfo)
       'http://dev.worldcookery.com/svn/bla/trunk/blergh'
  
+    You can use tags too
+
+      >>> def dummyinfo(path):
+      ...     return '''\
+      ... Path: .
+      ... URL: http://dev.worldcookery.com/svn/bla/branches/foobar/blergh
+      ... Repository UUID: ab69c8a2-bfcb-0310-9bff-acb20127a769
+      ... Revision: 1654
+      ... Node Kind: directory
+      ... '''
+
+      >>> determinebranch('tag/3.4', '.', svninfo=dummyinfo)
+      'http://dev.worldcookery.com/svn/bla/tag/3.4/blergh'
+
     """
     lines = svninfo(path).splitlines()
     if lines[1].startswith('URL: '):
@@ -206,18 +220,21 @@ def determinebranch(branch, path, svninfo=svninfo):
     chunks = url.split('/')
     chunks.reverse()
     new_chunks = []
+    is_tag = '/' in branch
 
     while chunks:
         ch = chunks.pop()
-        if ch in ('branch', 'branches'):
+        if ch in ('branch', 'branches', 'tag', 'tags'):
             chunks.pop()
             if branch == 'trunk':
                 new_chunks.append(branch)
             else:
-                new_chunks.append(ch)
+                if not is_tag:
+                    new_chunks.append(ch)
                 new_chunks.append(branch)
         elif ch == 'trunk' and branch != 'trunk':
-            new_chunks.append('branches')
+            if not is_tag:
+                new_chunks.append('branches')
             new_chunks.append(branch)
         else:
             new_chunks.append(ch)
@@ -272,10 +289,14 @@ def determinetag(tagname, path, svninfo=svninfo):
 
     while chunks:
         ch = chunks.pop()
-        if ch in ('branch', 'branches'):
+        if ch in ('branches', 'tags'):
             chunks.pop()
             new_chunks.append('tags')
             new_chunks.append(tagname)
+        elif ch in ('branch', 'tag'):
+            chunks.pop()
+            new_chunks.append('tag')
+            new_chunks.append(tagname)
         elif ch == 'trunk':
             new_chunks.append('tags')
             new_chunks.append(tagname)
@@ -322,10 +343,10 @@ def listbranches(path, svninfo=svninfo, svnls=svnls):
 
     while chunks:
         ch = chunks.pop()
-        if ch in ('branch', 'branches'):
-            chunks.append(ch)
+        if ch in ('tag', 'branch'):
+            chunks.append('branch')
             break
-        elif ch == 'trunk':
+        elif ch in ('tags', 'branches', 'trunk'):
             chunks.append('branches')
             break
 
@@ -336,6 +357,57 @@ def listbranches(path, svninfo=svninfo, svnls=svnls):
     return branches
 
 
+def listtags(path, svninfo=svninfo, svnls=svnls):
+    r"""
+    Let's set up a dummy 'svn info' command handler:
+
+      >>> def dummyinfo(path):
+      ...     return '''\
+      ... Path: .
+      ... URL: http://dev.worldcookery.com/svn/bla/trunk/blergh
+      ... Repository UUID: ab69c8a2-bfcb-0310-9bff-acb20127a769
+      ... Revision: 1654
+      ... Node Kind: directory
+      ... '''
+
+    and a dummy 'svn ls' as well:
+
+      >>> def dummyls(path):
+      ...     assert path == 'http://dev.worldcookery.com/svn/bla/tags'
+      ...     return '''\
+      ... foo/
+      ... README.txt
+      ... bar/
+      ... baz/
+      ... '''
+
+    ``listtags()`` takes the svn path of the current working
+    directory path, finds the URL of the repository, and lists all tags
+    in that repository.
+
+      >>> listtags('.', svninfo=dummyinfo, svnls=dummyls)
+      ['foo', 'bar', 'baz']
+ 
+    """
+    url = currentbranch(path, svninfo=svninfo)
+    chunks = url.split('/')
+
+    while chunks:
+        ch = chunks.pop()
+        if ch in ('trunk', 'branches', 'tags'):
+            chunks.append('tags')
+            break
+        elif ch in ('branch', 'tag'):
+            chunks.append('tag')
+            break
+
+    branches = []
+    for line in svnls('/'.join(chunks)).splitlines():
+        if line.endswith('/'):
+            branches.append(line[:-1])
+    return branches
+
+
 def branchpoints(branch, svnlog=svnlog):
     r"""
     Let's set up a dummy 'svn log' command handler:
@@ -534,7 +606,8 @@ def ezswitch(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
                 "usage: %prog [-n] [-c] [-m MSG] branch [wc-path]\n"
-                "       %prog -l\n"
+                "       %prog [-n] [-c] [-m MSG] -t tag [wc-path]\n"
+                "       %prog -l [-t]\n"
                 "       %prog",
                 prog=progname,
                 description="Switch the working directory to a different"
@@ -544,6 +617,9 @@ def ezswitch(argv, progname=None):
     parser.add_option('-l', '--list',
                       help='list existing branches',
                       action='store_true', dest='list_branches', default=False)
+    parser.add_option('-t', '--tag',
+                      help='look for a tag instead of a branch',
+                      action='store_true', dest='tag', default=False)
     parser.add_option('-c', '--create-branch',
                       help='create the new branch before switching to it',
                       action='store_true', dest='create_branch', default=False)
@@ -564,7 +640,10 @@ def ezswitch(argv, progname=None):
 
     if opts.list_branches:
         # TODO: allow a different wc-path
-        print '\n'.join(listbranches(path))
+        if opts.tag:
+            print '\n'.join(listtags(path))
+        else:
+            print '\n'.join(listbranches(path))
         return
 
     if not args:
@@ -572,6 +651,9 @@ def ezswitch(argv, progname=None):
         return
 
     branch = args[0]
+    if opts.tag:
+        branch = 'tags/' + branch
+
     if len(args) > 1:
         path = args[1]
 
@@ -604,6 +686,9 @@ def eztag(argv, progname=None):
     parser.add_option('-n', '--dry-run',
                       help='do not make the tag, just print the command',
                       action='store_true', dest='dry_run', default=False)
+    parser.add_option('-l', '--list',
+                      help='list existing tags',
+                      action='store_true', dest='list_tags', default=False)
     try:
         opts, args = parser.parse_args(argv[1:])
         if len(args) > 2:
@@ -614,6 +699,11 @@ def eztag(argv, progname=None):
     # TODO: allow a different wc-path
     path = '.'
 
+    if opts.list_tags:
+        # TODO: allow a different wc-path
+        print '\n'.join(listtags('.'))
+        return
+
     if len(args) < 1:
         parser.error("too few arguments, try %s --help" % progname)
     newtag = determinetag(args[0], path)
@@ -631,13 +721,16 @@ def ezbranch(argv, progname=None):
     progname = progname or os.path.basename(argv[0])
     parser = optparse.OptionParser(
                 "usage: %prog branch [wc-path]\n"
-                "       %prog -l\n"
+                "       %prog -l [-t]\n"
                 "       %prog",
                 prog=progname,
                 description="Print the URL of a named branch.")
     parser.add_option('-l', '--list',
                       help='list existing branches',
                       action='store_true', dest='list_branches', default=False)
+    parser.add_option('-t', '--tag',
+                      help='look for a tag instead of a branch',
+                      action='store_true', dest='tag', default=False)
     try:
         opts, args = parser.parse_args(argv[1:])
         if len(args) > 2:
@@ -649,7 +742,10 @@ def ezbranch(argv, progname=None):
 
     if opts.list_branches:
         # TODO: allow a different wc-path
-        print '\n'.join(listbranches(path))
+        if opts.tag:
+            print '\n'.join(listtags(path))
+        else:
+            print '\n'.join(listbranches(path))
         return
 
     if not args:
@@ -657,6 +753,9 @@ def ezbranch(argv, progname=None):
         return
 
     branch = args[0]
+    if opts.tag:
+        branch = 'tags/' + branch
+
     if len(args) > 1:
         path = args[1]
 
@@ -779,6 +878,33 @@ def branchdiff(argv, progname=None):
     os.system(diff_cmd)
 
 
+@command('branchpoint', 'show the revision number when a branch was created')
+def branchpoint(argv, progname=None):
+    progname = progname or os.path.basename(argv[0])
+    parser = optparse.OptionParser(
+                "usage: %prog [options] [branch [wc-path]]"
+                "       %prog -l\n",
+                prog=progname,
+                description="show the revision number when a branch was created")
+    parser.add_option('-l', '--list',
+                      help='list existing branches',
+                      action='store_true', dest='list_branches', default=False)
+    opts, args = parser.parse_args(argv[1:])
+    path = '.'
+    if opts.list_branches:
+        print '\n'.join(listbranches(path))
+        return
+    if args:
+        branch = args[0]
+        if len(args) > 1:
+            path = args[1]
+        branch = determinebranch(args[0], path)
+    else:
+        branch = currentbranch(path)
+    beginrev, endrev = branchpoints(branch)
+    print beginrev
+
+
 @command('selftest', 'run self-tests')
 def selftest(argv, progname=None):
     import doctest
